#include <debug.h>
#include <string.h>
#include "gs.h"
#include "gs_image.h"

#define DEBUG_GS_IMAGE

#ifdef DEBUG_GS_IMAGE
    #define DPRINT(fmt, ...) dprint(fmt, __VA_ARGS__)
#else
    #define DPRINT(fmt, ...)
#endif

static inline uint8* gs_image_get_built_in_data_cb(int *n, void *context);

/*
 * Put raw image to screen.
 *
 * ARGS
 *     x         horizontal position of image
 *     y         vertical   position of image
 *     width     width of image
 *     height    height of image
 *     tcolor    if not NULL substitute appropriate pixel in image with background color
 *     cb        callback function for data retrieve
 *     context   context passed to callback
 */
void gs_image_put(struct gs_layer_t *layer, int x, int y, int width, int height,
        union gs_pixel_t *tcolor, union gs_pixel_t *bgcolor,
        gs_image_getdata_cb cb, void *context)
{
    void *buf;
//    union gs_pixel_t *pixel;
    union gs_pixel_t *dst;
    union gs_pixel_t *src;
    int n, r, nx;

    if (!layer)
    {
        debug_emsg("NULL layer");
        return;
    }

    if (x + width > layer->width)
    {
        debug_emsg("image does not fit to layer by width");
        return;
    }
    if (y + height > layer->height)
    {
        debug_emsg("image does not fit to layer by width");
        return;
    }

    n = width * height * GS_PIXEL_DEPTH;


    dst = &layer->mem[y * layer->width + x];
    nx = width;

    /* XXX was splited to two parts (NULL/!NULL) for speed up? */
    if (tcolor == NULL || bgcolor == NULL)
    {
        while (n)
        {
            r = n;

            buf = (*cb)(&r, context);
            if (!buf || r == 0)
            {
                DEBUG_WMSG("callback return NULL");
                break;
            }
            
            if (r > n)
                r = n;
            n -= r;

            /* NOTE r should be multiple of GS_PIXEL_DEPTH */
            r /= GS_PIXEL_DEPTH;
            src = (union gs_pixel_t*)buf;
            while (r--)
            {
                (dst++)->value = (src++)->value;
                nx--;
                if (nx == 0)
                {
                    nx = width;
                    dst += layer->width - width;
                }
            }
        }
    } else {
        while (n)
        {
            r = n;

            buf = (*cb)(&r, context);
            if (!buf || r == 0)
            {
                DEBUG_WMSG("callback return NULL");
                break;
            }
            
            if (r > n)
                r = n;
            n -= r;

            /* NOTE r should be multiple of GS_PIXEL_DEPTH */
            r /= GS_PIXEL_DEPTH;
            src = (union gs_pixel_t*)buf;
            while (r--)
            {
                if (src->value == tcolor->value)
                    dst->value = bgcolor->value;
                else
                    dst->value = src->value;
                src++;
                dst++;
                nx--;
                if (nx == 0)
                {
                    nx = width;
                    dst += layer->width - width;
                }
            }
        }
    }
}

/*
 * Put built-in raw image to screen.
 */
void gs_image_put_built_in(struct gs_layer_t *layer, int x, int y,
        union gs_pixel_t *tcolor, union gs_pixel_t *bgcolor, void *data)
{
    struct gs_raw_image_t *image;

    if (!layer)
    {
        debug_emsg("NULL layer");
        return;
    }

    image = data;
    if (image->magic != BUILT_IN_IMAGE_MAGIC)
    {
        debug_emsg("invalid image magic number");
        return;
    }
    if (image->version != BUILT_IN_IMAGE_VERSION)
    {
        debug_emsg("unsupported version of image");
        return;
    }
    if (x + image->width > layer->width)
    {
        debug_emsg("image does not fit to layer by width");
        return;
    }
    if (y + image->height > layer->height)
    {
        debug_emsg("image does not fit to layer by height");
        return;
    }

    gs_image_put(layer, x, y, image->width, image->height, tcolor, bgcolor,
            gs_image_get_built_in_data_cb, image->data);
}

/*
 *
 */
static inline uint8* gs_image_get_built_in_data_cb(int *n, void *context)
{
    if (!n || *n == 0 || !context)
        return NULL;

    return context;
}

