#ifndef _GS_TEXT_H
#define _GS_TEXT_H

#include "gs.h"

/*
 *
 * Header format
 *
 *      0 -  1    0xBEEF
 *      2         header version
 *      3 -  4    header length
 *      5         number of pages
 *      6         maximum symbol width in pixels
 *      7         symbol heigth in pixels
 *      8 - 31    reserved
 *      32        page0   index
 *      33        page1   index
 *      ...
 *      xxx       page255 index
 *
 * Symbol format
 *      0        width of character
 *      1 - N    data
 * 
 *      5x7 symbol:
 *
 *          bit 01234567
 *
 *              xxxxxXXX
 *              xxxxxXXX
 *              xxxxxXXX
 *              xxxxxXXX
 *              xxxxxXXX
 *              xxxxxXXX
 *              xxxxxXXX
 *
 *      8x16 symbol
 *          bit 01234567 01234567
 *                               
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 *       byte 0 xxxxxxxx xxxxxxxx byte 1
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 *
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 *       byte 2 xxxxxxxx xxxxxxxx byte 3
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 *              xxxxxxxx xxxxxxxx
 */
struct gs_font_t {
#define GS_FONT_MAGIC   0xEFBE
    uint16 magic;
    uint8  version;
    uint16 hlen;
    uint8  npage;
    uint8  maxwidth; /* maximum width of pixel in bytes */
    uint8  height;
    uint8  reserved[24];
#define GS_FONT_PAGE_INDEX_COUNT    256
    uint8  ipage[GS_FONT_PAGE_INDEX_COUNT];
    /* NOTE flexible array member */
    uint8  data[];
} __attribute__((packed));

void gs_text_put(struct gs_layer_t *layer, char *text,int x, int y, int maxwidth,
        GS_COLOR fgcolor, GS_COLOR bgcolor, struct gs_font_t *font);

#endif

