/* 
 *     This file is part of K14, the LCD controller.
 * 
 * Copyright (C) 2014 Dmitry Kobylin
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/*
 *
 */
#include <LPC177x_8x.h>
#include <debug.h>
#include <uart.h>
#include <stimer.h>
#include <gpio.h>
#include "version.h"
#include "gs/gs.h"
#include "irqp.h"
#include "command.h"
#include "sdcard.h"
#include "media.h"

/*
 *
 */
int main(void)
{
    __enable_irq();

    irqp_init();
    gpio_init();
    stimer_init();
    uart_setup(DEBUG_UART, UART_BAUD_115200, NULL);

    dprint("nsn", "=================================");
    dprint("sn",  "= "MODULE_NAME);
    dprint("sn",  "=");
    dprint("sn",  "= v"VERSION_STRING", "__DATE__" "__TIME__);
    dprint("sn",  "=================================");

    gs_init();
    command_init();

    if (sdcard_init() < 0)
        debug_emsg("SD init error");
    else {
        if (media_init())
            debug_emsg("failed to init media");
    }


#if 1
    gs_text_put(gs_get_layer(GS_LAYER_BACK_BUFFER), "LCD Controller, sw. version " VERSION_STRING,
            10, 10, 0, GS_COLOR_YELLOW, GS_DEFAULT_COLOR, (struct gs_font_t*)gs_data_font_ter32b);

    /* XXX test that media is initialized */
    gs_cmd_put_jpeg_from_file(gs_get_layer(GS_LAYER_BACK_BUFFER), "/1.jpg", 36, 36);
    gs_cmd_flip();
#endif


    while (1)
    {
        command_process();
    }

    return 0;
}

