#include <debug.h>
#include "fat_io_lib/fat_filelib.h"
#include "sdcard.h"

#define DEBUG_MEDIA

#ifdef DEBUG_MEDIA
    #define DPRINT(fmt, ...)  dprint(fmt, __VA_ARGS__)
#else
    #define DPRINT(fmt, ...)
#endif

static int _listdir(char *path);
static int _read(uint32 sector, uint8 *buffer, uint32 sector_count);
static int _write(uint32 sector, uint8 *buffer, uint32 sector_count);



static FL_DIR _dirstat;
static struct fs_dir_ent _dirent;

/*
 *
 */
int media_init()
{
    if (fl_attach_media(_read, _write) != FAT_INIT_OK)
    {
        debug_emsg("media attach failed");
        return -1;
    }

    _listdir("/");

    return 0;
}

/*
 *
 */
void *media_open_file(char *path)
{
    FL_FILE *file;

    /* XXX readonly */
    file = (void*)fl_fopen(path, "r");
    if (!file)
        debug_emsgf("Failed to open file", "sn", path);

    return file;
}

/*
 *
 */
int media_read_file(void *file, char *buf, int len)
{
    int rd;

    rd = fl_fread(buf, len, 1, (FL_FILE*)file);
    if (rd < 0)
    {
        debug_emsg("File read error");
        rd = 0;
    }

    return rd;
}

/*
 *
 */
void media_seek(void *file, long offset, int origin)
{
    fl_fseek(file, offset, origin);
}

/*
 *
 */
void media_close_file(void *file)
{
    if (file)
        fl_fclose((FL_FILE*)file);
}

/*
 *
 */
static int _listdir(char *path)
{
    FL_DIR *dirstat;
    struct fs_dir_ent *dirent;

    dirstat = &_dirstat;
    dirent = &_dirent;

    DPRINT("ss*:n", "List content of ", path);
    if (fl_opendir(path, dirstat))
    {
        while (fl_readdir(dirstat, dirent) == 0)
        {
            if (dirent->is_dir)
                DPRINT("s", "<DIR  >");
            else
                DPRINT("s", "<FILE >");

            DPRINT("sn", dirent->filename);
        }

        fl_closedir(dirstat);
        return 0;
    }

    DEBUG_WMSGF("directory not found:", "s", path);
    return -1;
}


/******************************************************
 * high level functions for fat_io_lib
 ******************************************************/
static int _read(uint32 sector, uint8 *buffer, uint32 sector_count)
{
    while (sector_count--)
    {
        if (sdcard_read_block(sector++, buffer) < 0)
        {
            debug_emsg("sdcard read error");
            return 0;
        }
        buffer += 512;
    }

    return 1;
}

static int _write(uint32 sector, uint8 *buffer, uint32 sector_count)
{
    unsigned long i;

    dprint("sn", "media write not implemented");
    return 0;

    for (i = 0; i < sector_count; i++)
    {
        // ...
        // Add platform specific sector (512 bytes) write code here
        //..

        sector ++;
        buffer += 512;
    }

    return 1;
}

