#ifndef _SDCARD_DEF_H
#define _SDCARD_DEF_H

#include <types.h>

/************************************************
 * Physical Layer Simplified Specification v4.10. Tables 4-22 to 4-32.
 ***********************************************/
#define SD_CMD0_GO_IDLE_STATE        0    /* bc   response -   */
#define SD_CMD2_ALL_SEND_CID         2    /* bcr  response R2  */
#define SD_CMD3_SEND_RELATIVE_ADDR   3    /* bcr  response R6  */
#define SD_CMD7_SELECT_DESELECT_CARD 7    /* ac   response R1b */
#define SD_CMD8_SEND_IF_COND         8    /* bcr  response R7  */
#define SD_CMD9_SEND_CSD             9    /* ac   response R2  */
#define SD_CMD12_STOP_TRANSMISSION   12   /* ac   response R1b */
#define SD_CMD13_SEND_STATUS         13   /* ac   response R1  */
#define SD_CMD16_SET_BLOCKLEN        16   /* ac   response R1  */
#define SD_CMD17_READ_SINGLE_BLOCK   17   /* ac   response R1  */
#define SD_CMD18_READ_MULTIPLE_BLOCK 18   /* ac   response R1  */
#define SD_CMD55_APP_CMD             55   /* ac   response R1  */
#define SD_CMD58_READ_OCR            58   /* ac   response R3  */
#define SD_ACMD6_SET_BUS_WISTH       6    /* ac   response R1  */
#define SD_ACMD41_SD_SEND_OP_COND    41   /* bcr  response R3  */

#pragma pack(push, 1)
union sd_argument_t {
    uint32 value;

    struct {
        uint32 check_pattern    : 8;
#define VOLTAGE_ACCEPTED_27_36           0x01
#define VOLTAGE_ACCEPTED_RESERVED_LOW    0x02
        uint32 voltage_accepted : 4;
        uint32 reserved         : 20;
    } cmd08;

    struct {
        uint32 reserved   : 16;
        uint32 rca        : 16;
    } ac;

    /* OCR */
    struct {
        uint32 reserved00 : 15; /* 0 - 14 */
        uint32 v27_28     : 1;  /* 15 */
        uint32 v28_29     : 1;  /* 16 */
        uint32 v29_30     : 1;  /* 17 */
        uint32 v30_31     : 1;  /* 18 */
        uint32 v31_32     : 1;  /* 19 */
        uint32 v32_33     : 1;  /* 20 */
        uint32 v33_34     : 1;  /* 21 */
        uint32 v34_35     : 1;  /* 22 */
        uint32 v35_36     : 1;  /* 23 */
        uint32 s18r       : 1;  /* 24 */
        uint32 reserved01 : 3;  /* 25 - 27 */
        uint32 xpc        : 1;  /* 28   SDXC power control */ 
        uint32 fb         : 1;  /* 29 */
        uint32 hcs        : 1;  /* 30   Host Capacity Support, 0 - SDSC only, 1 - SDHC or SDXC */
        uint32 busy       : 1;  /* 31 */
    } acmd41;

    struct {
#define ACMD6_BUS_WIDTH_1    0
#define ACMD6_BUS_WIDTH_4    2
        uint32 bus_width  : 2; /* 0 - 1 */
        uint32 reserved   : 30;
    } acmd6;

    struct {
        uint32 reserved   : 16;
        uint32 rca        : 16;
    } cmd55;
};

union sd_response_t {
    uint8 val[5];

    struct {
        union {
            struct {
                uint8 in_idle_state       : 1; /* 0 */
                uint8 erase_reset         : 1; /* 1 */
                uint8 illegal_command     : 1; /* 2 */
                uint8 com_crc_error       : 1; /* 3 */
                uint8 erase_seq_error     : 1; /* 4 */
                uint8 address_error       : 1; /* 5 */
                uint8 parameter_error     : 1; /* 6 */
                uint8 zero                : 1; /* 7 */
            };
            uint8 val;
        } r1;

        union {
            struct {
                uint32 reserved00 : 15; /* 0 - 14 */
                uint32 v27_28     : 1;  /* 15 */
                uint32 v28_29     : 1;  /* 16 */
                uint32 v29_30     : 1;  /* 17 */
                uint32 v30_31     : 1;  /* 18 */
                uint32 v31_32     : 1;  /* 19 */
                uint32 v32_33     : 1;  /* 20 */
                uint32 v33_34     : 1;  /* 21 */
                uint32 v34_35     : 1;  /* 22 */
                uint32 v35_36     : 1;  /* 23 */
                uint32 s18a       : 1;  /* 24 */
                uint32 reserved01 : 4;  /* 25 - 28 */
                uint32 uhs2       : 1;  /* 29   0 - Non UHS-II card, 1 - UHS-II card */
                uint32 ccs        : 1;  /* 30   Card Capacity Status, 0 - SDSC, 1 - SDHC or SDXC */
                uint32 busy       : 1;  /* 31 */
            } r3;
            struct {
                uint8 card_is_locked      : 1; /* 0 */
                uint8 lock_unlock_failed  : 1; /* 1 */
                uint8 error               : 1; /* 2 */
                uint8 cc_error            : 1; /* 3 */
                uint8 card_ecc_failed     : 1; /* 4 */
                uint8 wp_violation        : 1; /* 5 */
                uint8 erase_param         : 1; /* 6 */
                uint8 out_of_range        : 1; /* 7 */
            } r2;
            struct {
                uint32 check_pattern    : 8;  /*  0 -  7 */
                uint32 voltage_accepted : 4;  /*  8 - 11 */
                uint32 reserved         : 16; /* 12 - 27 */
                uint32 command_version  : 4;  /* 28 - 31 */
            } r7;
            uint32 val16;
            uint32 val32;
        };
    };
};
#pragma pack(pop)

#endif /* !_SDCARD_DEF_H */

