#include <LPC177x_8x.h>
#include <LPC177x_8x_bits.h>
#include <debug.h>
#include <clk_cfg.h>
#include <stimer.h>
#include "sdram.h"

/* NOTE (?) SDRAM controller work on frequency <= 80 MHZ */

#define CLK_EMC_MHZ (CLK_EMC / 1000000)

#define tCLK_ns ((double)1000000000.0 / CLK_EMC)         // CCLK period in ns      
#define P2C(ns) ((uint32_t)( (double)(ns) / tCLK_ns ))   // convert ns to CCLKs

//#define ROW_BANK_COLUMN_MAP
#ifndef ROW_BANK_COLUMN_MAP
    #define BANK_ROW_COLUMN_MAP
#endif

#if 0
    /*
     * SDRAM Chip: MT48LC16M16A2P-75
     */
    #define SDRAM_RAS_LATENCY    3
    #define SDRAM_CAS_LATENCY    3
    
    /* 8192 cycles thru 64 ms ~ 7812 ns */
    #define SDRAM_ROWS            8192
    #define SDRAM_REFRESH_PERIOD  64     /* ms */
    
    #define SDRAM_REFRESH         (1000000 * SDRAM_REFRESH_PERIOD / SDRAM_ROWS)   /* ns */
    #define SDRAM_TRP             20     /* ns */
    #define SDRAM_TRAS            45     /* ns */
    #define SDRAM_TAPR            SDRAM_TRP      /* NOTE not tAPR value, using tRP value */
    #define SDRAM_TDAL            5      /* CLK */
    #define SDRAM_TWR             2      /* tWR, tDPL, tRWL or tRDL: CLK */
    #define SDRAM_TRC             66     /* ns */
    #define SDRAM_TRFC            66     /* ns */
    #define SDRAM_TXSR            75     /* ns */
    #define SDRAM_TRRD            15     /* ns */
    #define SDRAM_TMRD            3

#ifdef ROW_BANK_COLUMN_MAP
    #define DYNAMIC_CONFIG0 ((0 << 14) | (0 << 12) | (0x03 << 9) | (0x01 << 7))
#else
    #define DYNAMIC_CONFIG0 ((0 << 14) | (1 << 12) | (0x03 << 9) | (0x01 << 7))
#endif

#define SDRAM_COLUMN_BITS       9
#define SDRAM_ROW_BITS          13 
#define SDRAM_BANK_BITS         2
#define SDRAM_DATA_BITS         16

#endif /* 0/1 */

#if 0
    /*
     * SDRAM Chip: H57V256GTR-60C
     */
    #define SDRAM_RAS_LATENCY    3
    #define SDRAM_CAS_LATENCY    3
    
    /* 8192 cycles thru 64 ms ~ 7812 ns */
    #define SDRAM_ROWS            8192
    //#define SDRAM_REFRESH_PERIOD  64     /* ms */
    #define SDRAM_REFRESH_PERIOD  16     /* ms */
    
    #define SDRAM_REFRESH         (1000000 * SDRAM_REFRESH_PERIOD / SDRAM_ROWS)   /* ns */
    #define SDRAM_TRP             20         /* precharge command period, ns */
    #define SDRAM_TRAS            42         /* ns */
    #define SDRAM_TAPR            SDRAM_TRP  /* NOTE tRP value, last-data-out to active command time, ns */
    #define SDRAM_TDAL            2          /* CLK */
    #define SDRAM_TWR             2          /* tWR, tDPL, tRWL or tRDL, clocks */
    #define SDRAM_TRC             63         /* ns */
    #define SDRAM_TRFC            63         /* auto-refresh to active command, ns */
    #define SDRAM_TXSR            63         /* exit self refresh to active command, ns*/
    #define SDRAM_TSREX           15         /* self-refresh exit time, ns */
    #define SDRAM_TRRD            15         /* , ns */
    #define SDRAM_TMRD            3          /* , clocks */

#ifdef ROW_BANK_COLUMN_MAP
    #define DYNAMIC_CONFIG0 ((0 << 14) | (0 << 12) | (0x03 << 9) | (0x01 << 7))
#else
    #define DYNAMIC_CONFIG0 ((0 << 14) | (1 << 12) | (0x03 << 9) | (0x01 << 7))
#endif

#define SDRAM_COLUMN_BITS       9
#define SDRAM_ROW_BITS          13 
#define SDRAM_BANK_BITS         2
#define SDRAM_DATA_BITS         16

#endif /* 0/1 */


#if 0
    /*
     * SDRAM Chip: K4S561632J
     */
    #define SDRAM_RAS_LATENCY    3
    #define SDRAM_CAS_LATENCY    3
    
    /* 8192 cycles thru 64 ms ~ 7812 ns */
    #define SDRAM_ROWS            8192
    #define SDRAM_REFRESH_PERIOD  64     /* ms */
    //#define SDRAM_REFRESH_PERIOD  16     /* ms */
    
    #define SDRAM_REFRESH         (1000000 * SDRAM_REFRESH_PERIOD / SDRAM_ROWS)   /* ns */
    #define SDRAM_TRP             20         /* precharge command period, ns */
    #define SDRAM_TRAS            45         /* ns */
    #define SDRAM_TAPR            SDRAM_TRP  /* NOTE tRP value, last-data-out to active command time, ns */
    #define SDRAM_TDAL            2          /* CLK */
    #define SDRAM_TWR             2          /* tWR, tDPL, tRWL or tRDL, clocks */
    #define SDRAM_TRC             65         /* ns */
    #define SDRAM_TRFC            65         /* auto-refresh to active command, ns */
    #define SDRAM_TXSR            75         /* exit self refresh to active command, ns*/
    #define SDRAM_TSREX           15         /* self-refresh exit time, ns */
    #define SDRAM_TRRD            15         /* , ns */
    #define SDRAM_TMRD            3          /* , clocks */

#ifdef ROW_BANK_COLUMN_MAP
    #define DYNAMIC_CONFIG0 ((0 << 14) | (0 << 12) | (0x03 << 9) | (0x01 << 7))
#else
    #define DYNAMIC_CONFIG0 ((0 << 14) | (1 << 12) | (0x03 << 9) | (0x01 << 7))
#endif

#define SDRAM_COLUMN_BITS       9
#define SDRAM_ROW_BITS          13 
#define SDRAM_BANK_BITS         2
#define SDRAM_DATA_BITS         16

#endif /* 0/1 */

#if 1
    /*
     * SDRAM Chip: EM638165TS-6G
     */
    #define SDRAM_RAS_LATENCY    3
    #define SDRAM_CAS_LATENCY    3
    
    /* 8192 cycles thru 64 ms ~ 7812 ns */
    #define SDRAM_ROWS            8192
    #define SDRAM_REFRESH_PERIOD  64     /* ms */
    //#define SDRAM_REFRESH_PERIOD  16     /* ms */
    
    #define SDRAM_REFRESH         (1000000 * SDRAM_REFRESH_PERIOD / SDRAM_ROWS)   /* ns */
    #define SDRAM_TRP             20         /* precharge command period, ns */
    #define SDRAM_TRAS            45         /* ns */
    #define SDRAM_TAPR            SDRAM_TRP  /* NOTE tRP value, last-data-out to active command time, ns */
    #define SDRAM_TDAL            2          /* CLK */
    #define SDRAM_TWR             2          /* tWR, tDPL, tRWL or tRDL, clocks */
    #define SDRAM_TRC             65         /* ns */
    #define SDRAM_TRFC            65         /* auto-refresh to active command, ns */
    #define SDRAM_TXSR            75         /* exit self refresh to active command, ns*/
    #define SDRAM_TSREX           15         /* self-refresh exit time, ns */
    #define SDRAM_TRRD            15         /* , ns */
    #define SDRAM_TMRD            3          /* , clocks */

#ifdef ROW_BANK_COLUMN_MAP
    #define DYNAMIC_CONFIG0 ((0 << 14) | (0 << 12) | (0x01 << 9) | (0x01 << 7))
#else
    #define DYNAMIC_CONFIG0 ((0 << 14) | (1 << 12) | (0x01 << 9) | (0x01 << 7))
#endif

#define SDRAM_COLUMN_BITS       8
#define SDRAM_ROW_BITS          12 
#define SDRAM_BANK_BITS         2
#define SDRAM_DATA_BITS         16

#endif /* 0/1 */


static void wait_us(uint32 us);

/*
 * setup EMC to drive SDRAM
 */
void  __attribute__((optimize("O0"))) sdram_init()
{
    volatile unsigned long dummy;

    LPC_IOCON->P2_16 = 0x229;  /* #CAS    */
    LPC_IOCON->P2_17 = 0x229;  /* #RAS    */
    LPC_IOCON->P2_18 = 0x229;  /* CLK0    */
#if 0
    LPC_IOCON->P2_19 = 0x229;  /* CLK1    */
#endif
    LPC_IOCON->P2_20 = 0x229;  /* #DYCS0   */
#if 0
    LPC_IOCON->P2_21 = 0x229;  /* #DYCS1   */
    LPC_IOCON->P2_22 = 0x229;  /* #DYCS2   */
    LPC_IOCON->P2_23 = 0x229;  /* #DYCS3   */
#endif
    LPC_IOCON->P2_24 = 0x229;  /* CKE0    */
#if 0
    LPC_IOCON->P2_25 = 0x229;  /* CKE1    */
    LPC_IOCON->P2_26 = 0x229;  /* CKE2    */
    LPC_IOCON->P2_27 = 0x229;  /* CKE3    */
#endif
    LPC_IOCON->P2_28 = 0x229;  /* #DQM0   */
    LPC_IOCON->P2_29 = 0x229;  /* #DQM1   */
#if 0
    LPC_IOCON->P2_30 = 0x229;  /* #DQM2   */
    LPC_IOCON->P2_31 = 0x229;  /* #DQM3   */
#endif
    LPC_IOCON->P3_0  = 0x229;  /* D0      */
    LPC_IOCON->P3_1  = 0x229;  /* D1      */
    LPC_IOCON->P3_2  = 0x229;  /* D2      */
    LPC_IOCON->P3_3  = 0x229;  /* D3      */
    LPC_IOCON->P3_4  = 0x229;  /* D4      */
    LPC_IOCON->P3_5  = 0x229;  /* D5      */
    LPC_IOCON->P3_6  = 0x229;  /* D6      */
    LPC_IOCON->P3_7  = 0x229;  /* D7      */
    LPC_IOCON->P3_8  = 0x229;  /* D8      */
    LPC_IOCON->P3_9  = 0x229;  /* D9      */
    LPC_IOCON->P3_10 = 0x229;  /* D10     */
    LPC_IOCON->P3_11 = 0x229;  /* D11     */
    LPC_IOCON->P3_12 = 0x229;  /* D12     */
    LPC_IOCON->P3_13 = 0x229;  /* D13     */
    LPC_IOCON->P3_14 = 0x229;  /* D14     */
    LPC_IOCON->P3_15 = 0x229;  /* D15     */
#if 0
    LPC_IOCON->P3_16 = 0x229;  /* D16     */
    LPC_IOCON->P3_17 = 0x229;  /* D17     */
    LPC_IOCON->P3_18 = 0x229;  /* D18     */
    LPC_IOCON->P3_19 = 0x229;  /* D19     */
    LPC_IOCON->P3_20 = 0x229;  /* D20     */
    LPC_IOCON->P3_21 = 0x229;  /* D21     */
    LPC_IOCON->P3_22 = 0x229;  /* D22     */
    LPC_IOCON->P3_23 = 0x229;  /* D23     */
    LPC_IOCON->P3_24 = 0x229;  /* D24     */
    LPC_IOCON->P3_25 = 0x229;  /* D25     */
    LPC_IOCON->P3_26 = 0x229;  /* D26     */
    LPC_IOCON->P3_27 = 0x229;  /* D27     */
    LPC_IOCON->P3_28 = 0x229;  /* D28     */
    LPC_IOCON->P3_29 = 0x229;  /* D29     */
    LPC_IOCON->P3_30 = 0x229;  /* D30     */
    LPC_IOCON->P3_31 = 0x229;  /* D31     */
#endif
    LPC_IOCON->P4_0  = 0x229;  /* A0      */
    LPC_IOCON->P4_1  = 0x229;  /* A1      */
    LPC_IOCON->P4_2  = 0x229;  /* A2      */
    LPC_IOCON->P4_3  = 0x229;  /* A3      */
    LPC_IOCON->P4_4  = 0x229;  /* A4      */
    LPC_IOCON->P4_5  = 0x229;  /* A5      */
    LPC_IOCON->P4_6  = 0x229;  /* A6      */
    LPC_IOCON->P4_7  = 0x229;  /* A7      */
    LPC_IOCON->P4_8  = 0x229;  /* A8      */
    LPC_IOCON->P4_9  = 0x229;  /* A9      */
    LPC_IOCON->P4_10 = 0x229;  /* A10     */
    LPC_IOCON->P4_11 = 0x229;  /* A11     */
    LPC_IOCON->P4_12 = 0x229;  /* A12     */
    LPC_IOCON->P4_13 = 0x229;  /* A13     */
    LPC_IOCON->P4_14 = 0x229;  /* A14     */
#if 0
    LPC_IOCON->P4_15 = 0x229;  /* A15     */
    LPC_IOCON->P4_16 = 0x229;  /* A16     */
    LPC_IOCON->P4_17 = 0x229;  /* A17     */
    LPC_IOCON->P4_18 = 0x229;  /* A18     */
    LPC_IOCON->P4_19 = 0x229;  /* A19     */
    LPC_IOCON->P4_20 = 0x229;  /* A20     */
    LPC_IOCON->P4_21 = 0x229;  /* A21     */
    LPC_IOCON->P4_22 = 0x229;  /* A22     */
    LPC_IOCON->P4_23 = 0x229;  /* A23     */
#endif
    LPC_IOCON->P4_25 = 0x229;  /* #WE     */

    /* Power up SDRAM controller */
    LPC_SC->PCONP |= PCONP_PCEMC;

    /*
     * Set dynamic memory read strategy.
     *
     *
     * http://www.lpcware.com/content/forum/lpc177x-emc-and-sdram-conundrum
     *
     *     1. EMC clock <= 60 MHz: clock delayed read strategy.
     *     CMDDLY = 0, CLKOUT0DLY = 31, FBCLKDLY = 16. Fastest option.
     *     
     *     2. EMC clock <= 80 MHz: command delayed read strategy + 0.
     *     CLKOUT0DLY = 0, FBCLKDLY = 20. CMDDLY in relation to EMC clock!
     *     CMDDLY = 31 for EMCCLK <= 48 MHz,
     *     CMDDLY = 27 for EMCCLK  =60 MHz,
     *     CMDDLY = 16 for EMCCLK  =72 MHz.
     *     
     *     3. EMC clock >= 80 MHz: command delayed read strategy + 1.
     *     CLKOUT0DLY = 0, FBCLKDLY = 20. CMDDLY = 2. (Not valid area, but working)
     */
    {
        uint32 delay;

#define SDRAM_READ_CONFIG_CLKOUT_DELAY    0x00   /* Clock out delayed strategy, using CLKOUT (command not delayed, clock out delayed) */
#define SDRAM_READ_CONFIG_COMMAND_DELAY   0x01   /* Command delayed strategy, using EMCCLKDELAY (command delayed, clock out) not delayed) */
#define SDRAM_READ_CONFIG_COMMAND_DELAY1  0x02   /* Command delayed strategy plus one clock cycle, using EMCCLKDELAY (command delayed, clock out) not delayed) */
#define SDRAM_READ_CONFIG_COMMAND_DELAY2  0x03   /* Command delayed strategy plus two clock cycles, using EMCCLKDELAY (command delayed, clock out) not delayed) */

#if (CLK_EMC_MHZ <= 60)
        #define SDRAM_READ_CONFIG SDRAM_READ_CONFIG_CLKOUT_DELAY

        delay = ( 0 <<  0) | /*  0 :  4, CMDDLY     */
                (16 <<  8) | /*  8 : 12, FBCLKDLY   */
                (31 << 16) | /* 16 : 20, CLKOUT0DLY */
                ( 0 << 24);  /* 24 : 28, CLKOUT1DLY */
#elif (CLK_EMC_MHZ <= 80)
        #define SDRAM_READ_CONFIG SDRAM_READ_CONFIG_COMMAND_DELAY

        delay = (16 <<  0) | /*  0 :  4, CMDDLY     */
                (20 <<  8) | /*  8 : 12, FBCLKDLY   */
                ( 0 << 16) | /* 16 : 20, CLKOUT0DLY */
                ( 0 << 24);  /* 24 : 28, CLKOUT1DLY */
#elif (CLK_EMC_MHZ > 80)
        #define SDRAM_READ_CONFIG SDRAM_READ_CONFIG_COMMAND_DELAY1

        delay = ( 2 <<  0) | /*  0 :  4, CMDDLY     */
                (20 <<  8) | /*  8 : 12, FBCLKDLY   */
                ( 0 << 16) | /* 16 : 20, CLKOUT0DLY */
                ( 0 << 24);  /* 24 : 28, CLKOUT1DLY */
#else
        #error Unknown EMC clock frequency
#endif

        LPC_SC->EMCDLYCTL = delay;
    }

    LPC_EMC->Control           = 1;

    LPC_EMC->DynamicReadConfig = SDRAM_READ_CONFIG;
    LPC_EMC->DynamicRasCas0    = (SDRAM_RAS_LATENCY << 0) | (SDRAM_CAS_LATENCY << 8);
    LPC_EMC->DynamicRP         = P2C(SDRAM_TRP);
    LPC_EMC->DynamicRAS        = P2C(SDRAM_TRAS);
    LPC_EMC->DynamicSREX       = P2C(SDRAM_TXSR);
    LPC_EMC->DynamicAPR        = P2C(SDRAM_TAPR);
    LPC_EMC->DynamicDAL        = SDRAM_TDAL + P2C(SDRAM_TRP) + 1;
    LPC_EMC->DynamicWR         = SDRAM_TWR - 1;
    LPC_EMC->DynamicRC         = P2C(SDRAM_TRC);
    LPC_EMC->DynamicRFC        = P2C(SDRAM_TRFC);
    LPC_EMC->DynamicXSR        = P2C(SDRAM_TXSR);
    LPC_EMC->DynamicRRD        = P2C(SDRAM_TRRD);
    LPC_EMC->DynamicMRD        = SDRAM_TMRD - 1;

    /* 13 row, 9 - col */
    LPC_EMC->DynamicConfig0 = DYNAMIC_CONFIG0;

    /*
     * JEDEC General SDRAM Initialization Sequence delay to allow
     * power and clocks to stabilize ~100 us
     *
     * NOP
     */
    LPC_EMC->DynamicControl = 0x0183;
    wait_us(200);
    /* PALL */
    LPC_EMC->DynamicControl = 0x0103;
    LPC_EMC->DynamicRefresh = 1;
    wait_us(50);
    LPC_EMC->DynamicRefresh = (P2C(SDRAM_REFRESH)) >> 4;
    /* COMM */
    LPC_EMC->DynamicControl = 0x00000083; /* Issue MODE command */


#define SDRAM_SEQUENTIAL_BURST_LENGTH_4    2
#define SDRAM_SEQUENTIAL_BURST_LENGTH_8    3

#define SDRAM_BURST_LENGTH SDRAM_SEQUENTIAL_BURST_LENGTH_8

#define SDRAM_MODE_REG_VALUE    ((SDRAM_CAS_LATENCY << 4) | SDRAM_BURST_LENGTH)

#ifdef ROW_BANK_COLUMN_MAP
    dummy = *((volatile uint32_t *)(SDRAM_BASE_ADDR |
            (SDRAM_MODE_REG_VALUE << (SDRAM_COLUMN_BITS + SDRAM_BANK_BITS + SDRAM_DATA_BITS / 16))));
#else
    dummy = *((volatile uint32_t *)(SDRAM_BASE_ADDR |
            (SDRAM_MODE_REG_VALUE << (SDRAM_COLUMN_BITS + SDRAM_DATA_BITS / 16))));
#endif

    (void)dummy; /* make compiller happy */

    /* NORM */
    LPC_EMC->DynamicControl = 0x0000;

    /* enable buffer */
    LPC_EMC->DynamicConfig0 |= (1 << 19);
    wait_us(200);

    /*
     * print result timings
     */
    {
        uint32 val;

#define SCALE  10
#define CLK2NS(clk)     (SCALE * 1000 / (CLK_EMC / 1000000) * (clk))
        dprint("sn", "SDRAM timing: ");
        dprint("<    CLK_EMC, MHz                                           >4dn", CLK_EMC / 1000000);
        dprint("<    CLK_EMC, ns                                            >4d*.4dn", CLK2NS(1) / SCALE, CLK2NS(1) % SCALE);

        val = (LPC_EMC->DynamicRefresh & 0x3ff) << 4;
        dprint("<    Refresh, ns                                            >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        dprint("<    RAS latency, clk                                       >4dn", (LPC_EMC->DynamicRasCas0 >> 0) & 0x03);
        dprint("<    CAS latency, clk                                       >4dn", (LPC_EMC->DynamicRasCas0 >> 8) & 0x03);

        val = (LPC_EMC->DynamicRP & 0x0f) + 1;
        dprint("<    tRP   (precharge command period), ns                   >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        val = (LPC_EMC->DynamicRAS & 0x0f) + 1;
        dprint("<    tRAS  (active to precharge command period), ns         >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        val = (LPC_EMC->DynamicSREX & 0x0f) + 1;
        dprint("<    tSREX (self-refresh exit time), ns                     >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        val = (LPC_EMC->DynamicAPR & 0x0f) + 1;
        dprint("<    tAPR  (last-data-out to active command time), ns       >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        val = (LPC_EMC->DynamicDAL & 0x0f);
        dprint("<    tDAL  (data-in to active command time), clk            >4dn", val);
        val = (LPC_EMC->DynamicWR & 0x0f) + 1;
        dprint("<    tWR   (write recovery time), ns                        >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        val = (LPC_EMC->DynamicRC & 0x0f) + 1;
        dprint("<    tRC   (active to active command period), ns            >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        val = (LPC_EMC->DynamicRFC & 0x1f) + 1;
        dprint("<    tRFC  (auto-refresh period), ns                        >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        val = (LPC_EMC->DynamicXSR & 0x1f) + 1;
        dprint("<    tXSR  (exit self-refresh to active command time), ns   >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        val = (LPC_EMC->DynamicRRD & 0x0f) + 1;
        dprint("<    tRRD  (active bank A to active bank B latency), ns     >4d*.4dn", CLK2NS(val) / SCALE, CLK2NS(val) % SCALE);
        val = (LPC_EMC->DynamicMRD & 0x0f) + 1;
        dprint("<    tMRD  (load mode register to active command time), clk >4dn", val);
    }
}

/*
 * RETURN
 *     1 on success, 0 on error
 */
int  __attribute__((optimize("O0"))) sdram_test(int type)
{
    int ntest;
    uint32 tm;
    volatile uint8 *p8;
    volatile uint16 *p16;
    volatile uint32 *p32;
    uint16 val;
    uint32 bank, row, column, n, i;
    uint32 mask;

    for (ntest = 0; ntest < 2; ntest++)
    {
        dprint("<SDRAM test #>1d*:n", ntest);

#ifndef ROW_BANK_COLUMN_MAP
        if (type == SDRAM_TEST_SHORT)
        {
            /*
             * Test each line of address (each row, each bank)
             */
            dprint("ts", "short test... ");
            stimer_settime_us(&tm);

            p16 = (volatile uint16*)SDRAM_BASE_ADDR;
            for (bank = 0; bank < (1 << SDRAM_BANK_BITS); bank++)
            {
                for (row = 0; row < (1 << SDRAM_ROW_BITS); row++)
                {
                    *p16 = row | (bank << 14);
                    p16 += (1 << SDRAM_COLUMN_BITS);
                }
            }

            p16 = (volatile uint16*)SDRAM_BASE_ADDR;
            for (bank = 0; bank < (1 << SDRAM_BANK_BITS); bank++)
            {
                for (row = 0; row < (1 << SDRAM_ROW_BITS); row++)
                {
                    if (*p16 != (row | (bank << 14)))
                    {
                        dprint("sn", "FAILED");
                        dprint("tt<row  >4xn", row);
                        dprint("tt<bank >4xn", bank);
                        return 0;
                    }
                    p16 += (1 << SDRAM_COLUMN_BITS);
                }
            }

            tm = stimer_deltatime_us(tm);
            dprint("<done (>4d*.4d< ms)>n", tm / 1000, tm % 1000);
        } else if (type == SDRAM_TEST_FULL) {
            /*
             * Clean-up memory
             */
            dprint("ts", "clean-up memory         ");
            stimer_settime_us(&tm);
            p32 = (volatile uint32*)SDRAM_BASE_ADDR;
            for (i = 0; i < (SDRAM_SIZE / 4); i++)
            {
                *p32 = 0;
                if (*p32)
                {
                    dprint("sn", "FAILED");
                    dprint("tt<addr  >4xn", p32);
                }
                p32++;
            }
            tm = stimer_deltatime_us(tm);
            dprint("<done (>4d*.4d< ms)>n", tm / 1000, tm % 1000);

            dprint("ts", "running bit             ");
            stimer_settime_us(&tm);
            p16 = (volatile uint16*)SDRAM_BASE_ADDR;
            for (mask = 1; mask < 0x10000; mask <<= 1)
                *p16++ = mask & 0xffff;

            p16 = (volatile uint16*)SDRAM_BASE_ADDR;
            for (mask = 1; mask < 0x10000; mask <<= 1)
            {
                if (*p16++ != mask)
                {
                    dprint("sn", "FAILED");
                    dprint("tt<mask  >4xn", mask);
                    return 0;
                }
            }

            tm = stimer_deltatime_us(tm);
            dprint("<done (>4d*.4d< ms)>n", tm / 1000, tm % 1000);

            /*
             * Sequential write all memory locations by 16-bit
             */
            dprint("ts", "16-bit write            ");
            stimer_settime_us(&tm);
            n = 0;
            p16 = (volatile uint16*)SDRAM_BASE_ADDR;
            for (bank = 0; bank < (1 << SDRAM_BANK_BITS); bank++)
            {
                for (row = 0; row < (1 << SDRAM_ROW_BITS); row++)
                {
                    for (column = 0; column < (1 << SDRAM_COLUMN_BITS); column++)
                    {
                        *p16 = n & 0xffff;
                        p16++;
                        n++;
                    }
                    n++;
                }
            }
            n = 0;

            p16 = (volatile uint16*)SDRAM_BASE_ADDR;
            for (bank = 0; bank < (1 << SDRAM_BANK_BITS); bank++)
            {
                for (row = 0; row < (1 << SDRAM_ROW_BITS); row++)
                {
                    for (column = 0; column < (1 << SDRAM_COLUMN_BITS); column++)
                    {
                        val = *p16;
                        if (val != (n & 0xffff))
                        {
                            dprint("sn", "FAILED");
                            dprint("tt<addr   >4xn", p16);
                            dprint("tt<col    >4xn", column);
                            dprint("tt<row    >4xn", row);
                            dprint("tt<bank   >4xn", bank);
                            dprint("tt<write  >2xn", n);
                            dprint("tt<read   >2xn", val);
                            return 0;
                        }
                        p16++;
                        n++;
                    }
                    n++;
                }
            }
            tm = stimer_deltatime_us(tm);
            dprint("<done (>4d*.4d< ms)>n", tm / 1000, tm % 1000);

            /*
             * Sequential write all memory locations by 8-bit
             */
            dprint("ts", "8-bit write             ");
            stimer_settime_us(&tm);
            n = 0;
            p8 = (volatile uint8*)SDRAM_BASE_ADDR;
#define SDRAM_8BIT_PATTERN ((i ^ ( (i / 256) % 2 ? 0xff : 0)) & 0xff)
            for (i = 0; i < SDRAM_SIZE; i++)
                *p8++ = SDRAM_8BIT_PATTERN;

            p8 = (volatile uint8*)SDRAM_BASE_ADDR;
            for (i = 0; i < SDRAM_SIZE; i++)
            {
                if (*p8 != SDRAM_8BIT_PATTERN)
                {
                    dprint("sn", "FAILED");
                    dprint("tt<addr    >4xn", p8);
                    dprint("tt<pattern >1xn", SDRAM_8BIT_PATTERN);
                    return 0;
                }
                p8++;
            }
            tm = stimer_deltatime_us(tm);
            dprint("<done (>4d*.4d< ms)>n", tm / 1000, tm % 1000);
        }
#endif
    }

    return 1;
}


/*
 * delay approximately for N microseconds interval
 */
void  __attribute__((optimize("O0"))) wait_us(uint32 us)
{
    uint32 nclk;

    nclk = CLK_CCLK / 1000000 * us;

    while (nclk--)
    {
        asm volatile("nop");
    }
}

