#ifndef _DEBUG_H_
#define _DEBUG_H_

#include <stdio.h>
#include <stdarg.h>
#include <types.h>

#define ERR_PREFIX    "(E) "
#define WARN_PREFIX   "(W) "

int vudprint(char *dst, char *fmt, va_list va);
int udprint(char *dst, char *fmt, ...);

#ifdef USE_DEBUG
    #define dprint(fmt, ...) udprint(NULL, fmt, __VA_ARGS__)
#else
    #define dprint(fmt, ...)
#endif

#define sdprint(dst, fmt, ...) udprint(dst, fmt, __VA_ARGS__)

#define DEBUG_IMSG(msg) \
            DPRINT("s*:_sn", __FUNCTION__, msg)
#define DEBUG_IMSGF(msg, fmt, ...)                             \
        do                                                     \
        {                                                      \
            DPRINT("s*:_s_", __FUNCTION__, msg);               \
            DPRINT(fmt, __VA_ARGS__);                          \
        } while (0)

#define DEBUG_WMSG(msg) \
            DPRINT("ss*:_sn", WARN_PREFIX, __FUNCTION__, msg)
#define DEBUG_WMSGF(msg, fmt, ...)                             \
        do                                                     \
        {                                                      \
            DPRINT("ss*:_s_", WARN_PREFIX, __FUNCTION__, msg); \
            DPRINT(fmt, __VA_ARGS__);                          \
        } while (0)

#define DEBUG_EMSG(msg) \
            DPRINT("ss*:_sn", ERR_PREFIX, __FUNCTION__, msg)
#define DEBUG_EMSGF(msg, fmt, ...)                            \
        do                                                     \
        {                                                     \
            DPRINT("ss*:_s_", ERR_PREFIX, __FUNCTION__, msg); \
            DPRINT(fmt, __VA_ARGS__);                         \
        } while (0)

#define debug_emsg(msg) \
            dprint("ss*:_sn", ERR_PREFIX, __FUNCTION__, msg)
#define debug_emsgf(msg, fmt, ...)                            \
        do                                                    \
        {                                                     \
            dprint("ss*:_s_", ERR_PREFIX, __FUNCTION__, msg); \
            dprint(fmt, __VA_ARGS__);                         \
        } while (0)

#endif /* !_DEBUG_H_ */

