#ifndef LPC17XX_SSP_H
#define LPC17XX_SSP_H

#include <types.h>
#include <LPC177x_8x.h>
#include <clk_cfg.h>

struct ssp_t {
    LPC_SSP_TypeDef *pm;

    uint32 speed;
    uint32 mode;
};

#define SSP_COUNT 2

extern struct ssp_t ssp[SSP_COUNT];

#define SSP0 (&ssp[0])
#define SSP1 (&ssp[1])

/*
 * Freq = PCLK / (CPSDVSR * (SCR + 1))
 *
 * speed bits:
 *     0..7    CPSDVSR  (even values between 2 and 254)
 *     8..15   SCR
 */
#define SSP_SPEED_100KHZ_24MHZ   (80 | (2 << 8))
#define SSP_SPEED_6MHZ_24MHZ     (2  | (1 << 8))
#define SSP_SPEED_12MHZ_24MHZ    (2  | (0 << 8))

#define SSP_SPEED_5MHZ_20MHZ     (2  | (1 << 8))
#define SSP_SPEED_10MHZ_20MHZ    (2  | (0 << 8))

#if CLK_PCLK   == (24 * 1000000)
    #define SSP_SPEED_100KHZ   SSP_SPEED_100KHZ_24MHZ
    #define SSP_SPEED_6MHZ     SSP_SPEED_6MHZ_24MHZ
    #define SSP_SPEED_12MHZ    SSP_SPEED_12MHZ_24MHZ
#elif CLK_PCLK == (20 * 1000000)                 
    #define SSP_SPEED_5MHZ     SSP_SPEED_5MHZ_20MHZ
    #define SSP_SPEED_10MHZ    SSP_SPEED_10MHZ_20MHZ
#else
    #error Unknown PCLK frequency
#endif



/*
 * bit 0    CPHA
 * bit 1    CPOL
 */
#define SSP_MODE_CPOL1_CPHA1   0x03
#define SSP_MODE_CPOL0_CPHA1   0x01
#define SSP_MODE_CPOL1_CPHA0   0x02
#define SSP_MODE_CPOL0_CPHA0   0x00

void ssp_setup(struct ssp_t *sspif, uint32 speed, uint32 mode);
void ssp_setmode(struct ssp_t *sspif, uint32 mode);
void ssp_setspeed(struct ssp_t *sspif, uint32 speed);
void ssp_wr(struct ssp_t *sspif, uint8 *tx, uint8 *rx, uint32 len);
void ssp_disable(struct ssp_t *sspif);

#endif

