#!/bin/sh
#\
exec tclsh "$0" "$@"

proc getfiles {dir pat} {
    set files [list]
    foreach file [glob -nocomplain -directory $dir *] {
        if {[file isdirectory $file]} {
            lappend files {*}[getfiles $file $pat]
        } elseif {[string match $pat [file tail $file]]} {
            lappend files $file
        }
    }

    return $files
}

if {$argc >= 1} {
    set idx 0
    foreach arg $argv {
        if {$arg eq "-h"} {
            puts "Usage:"
            puts "    [file tail [info script]] <-g GLOB_PATTERN> <-p PAGES>"
            puts ""
            exit 1
        }
        if {$arg eq "-g"} {
            set pat [lindex $argv [incr idx]]
            continue
        }
        if {$arg eq "-p"} {
            set PAGES [lrange $argv [incr idx] end]
            break
        }
        incr idx
    }
} else {
    set PAGES [list]
}

if {![info exists pat]} {
    set pat *.bdf
}

set fonts [getfiles fonts $pat]

set OUTDIR /dev/shm/output
file mkdir $OUTDIR

foreach font $fonts {
    set dst_pbm [file join $OUTDIR [string range [file tail $font] 0 end-4].pbm]
    set dst_bin [file join $OUTDIR [string range [file tail $font] 0 end-4].bin]

    puts -nonewline "process [set font]... "
    flush stdout
    if {[catch {
        exec ./bdf2raw.tcl $font $dst_pbm {*}$PAGES
        exec ./bdf2raw.tcl $font $dst_bin {*}$PAGES
    } result]} {
        puts error
        puts $result
        exit 1
    } else {
        puts done
    }
}

