#!/bin/env lua

if #arg < 2 then
    print("Usage:")
    print("",  arg[0] .. " source.pnm dest.bin")
    print("",  arg[0] .. " source.pnm dest.pnm")
    print()
    os.exit(1)
end

source = arg[1]
dest   = arg[2]

if string.match(dest, ".*%.bin") then
    format = "bin"
elseif string.match(dest, ".*%.pnm") then
    format = "pnm"
else
    print("Invalid output format")
    os.exit(1)
end

ifd = io.open(source, "rb");
ofd = io.open(dest,   "wb");

line = ifd:read("*l")
if line ~= "P6" then
    print("Invalid input file format")
    os.exit(1)
end

while true do
    ::next::
    line = ifd:read("*l")
    if string.match(line, "^#") then
        goto next
    end

    if not (width and height) then
        width, height = string.match(line, "(%d+)%s+(%d+)")
    else
        colormax      = string.match(line, "(%d+)")
        break
    end
end

data = ifd:read("*a")

print("Width    :", width)
print("Height   :", height)
print("Colormax :", colormax)

if not (width and height and colormax and data) then
    print("Invalid input file format")
    os.exit(1)
end

--function rgb2yuv(r, g, b)
--    local y, u, v
--
--    y = ( (257 * r) + (504 * g) + ( 98 * b)) / 1000 + 16
--    u = (-(148 * r) - (291 * g) + (439 * b)) / 1000 + 128
--    v = ( (439 * r) - (368 * g) - ( 71 * b)) / 1000 + 128
--
--    return y, u, v
--end
--
--function yuv2rgb(y, u, v)
--    local r, g, b
--
--    r = (1164 * (y - 16) + 1596 * (v - 128)                   ) / 1000;
--    g = (1164 * (y - 16) -  813 * (v - 128) -  391 * (u - 128)) / 1000;
--    b = (1164 * (y - 16)                    + 2018 * (u - 128)) / 1000;
--
--    return normalize(r, g, b, 255)
--end
--
--function normalize(r, g, b, max)
--    if (r > max) then r = max end; if (r < 0) then r = 0 end
--    if (g > max) then g = max end; if (g < 0) then g = 0 end
--    if (b > max) then b = max end; if (b < 0) then b = 0 end
--    
--    return r, g, b
--end


if format == "pnm" then
    ofd:write("P6\n")
    ofd:write(string.format("%u %u\n", width, height))
    ofd:write(string.format("%u\n", 32))

    gwidth = 5
else
    ofd:write(string.format("%c%c", 0x49, 0x4D))
    version = 1
    ofd:write(string.format("%c%c", version, 0))
    ofd:write(string.format("%c%c", width,  bit32.rshift(width, 8)))
    ofd:write(string.format("%c%c", height, bit32.rshift(height, 8)))

    gwidth = 6
end


for i = 1, #data, 3 do
    local y, u, v
    -- transform to 565

    r = tonumber(string.format("%u", data:byte(i + 0)));
    g = tonumber(string.format("%u", data:byte(i + 1)));
    b = tonumber(string.format("%u", data:byte(i + 2)));

    r = bit32.rshift(r, 8 - 5)
    g = bit32.rshift(g, 8 - gwidth)
    b = bit32.rshift(b, 8 - 5)

    if format == "bin" then
        word = bit32.bor(r, bit32.lshift(g, 5), bit32.lshift(b, 11))
        ofd:write(string.format("%c%c", 
            bit32.band(word, 0xff), bit32.band(bit32.rshift(word, 8), 0xff)
        ))
    else
        ofd:write(string.format("%c%c%c", r, g, b))
    end
end

ifd:close()
ofd:close()

